c     Program continuous_recharge.f: calls Nag routine to solve a continuous
c     recharge equation similar to Eq (3.1) in Jin 1997, part I.
c     Altered by Fiona to use mm_standard.* instead of continuous_ver3
c     Altered to do output of temp + h only and at a day resolution
c     instead of a month.
c     Changed weights, line 192

c---------------------------------
c     DELAYED OSCILLATOR VERSION |
c---------------------------------
      include 'mm_standard.h'
      character ll
      EXTERNAL FCN_nag, OUTPUT_nag
      integer  tt, ii,itau1,itau2
      real*8 pi,t_middle,tday,Yn,a_plus,a_minus,h_plus,h_minus
      real*8 k,b_plus,b_minus,T1,T2,b1,b2,H0
      common/Tsub_coef/ k,b_plus,b_minus,a_plus,a_minus,h_plus,h_minus
      double precision x_rand, rand
      real*8 forcing_correlation_time,Rforce,Rforce_s,dt_m
      real*8 D02CJW
      external D02CJW

c     ------------------
c     begin main program
c     ------------------

c     open input file, read parameters:
c     ---------------------------------
      open(unit=5,file='mm_standard.dat',status='old')
      read(5,nl_pars)
      open(unit=10,file='mm_standard.out',form='formatted')
c      open(unit=20,file='mm_standard.test',form='formatted')
c      open(unit=50,file='Markov.out',form='formatted')

c     random number generator with Marcov correlation
c     -----------------------------------------------
      dt_m=1./30.
      forcing_correlation_time = 1./6.
      Rforce=exp(-dt_m/forcing_correlation_time)
      Rforce_s = sqrt(1.d0-Rforce**2)
      do ii=2,300000
         x_rand=rand()
         random_amp(ii)=Rforce*random_amp(ii-1)+Rforce_s*(x_rand-.5)
cc         write(50,'(f10.4)')random_amp(ii)
      enddo


c     define a seasonal varing kappa function as a function of days:
c     --------------------------------------------------------------
      call coupling()

c     set parameters:
c     ---------------
      first=120
      last=first+last_year*12
      betta=5.96E-5
      ru=1000
      L=1.5E7
      Lo=sqrt(Co/betta)
      Yn=dY*Lo
      tau1=L*Yn*Yn*betta/(Co*Co)
      tau2=L/Co
      A=exp(-.05*dY*dY)*(1+.1*dY*dY)/(Yn*Yn)
      em21=exp(    -tau2*em)
      em22=exp(-0.5*tau2*em)
      em11=exp(    -tau1*em)
      em12=exp(-0.5*tau1*em)
      itau1=tau1
      itau2=tau2
      delay1=(tau2+tau1/2)
      delay2=(tau2/2)
      delay3=(tau2)
      delay4=(tau2+tau1)

c set parameters for Tsub
c------------------------
      T1=28.
      T2=-40.
      b1=1./(80.)
      b2=1./(33.)
      H0=47.
      k=0.5*( b1*T1/(cosh(b1*H0)*cosh(b1*H0))
     *       -b2*T2/(cosh(b2*H0)*cosh(b2*H0)) )
      b_plus=T1*(1.-tanh(b1*H0))
      b_minus=T2*(1.-tanh(b2*H0))
      a_plus=1
      a_minus=1
      h_plus=b_plus*(a_plus-1)/a_plus
      h_minus=-b_minus*(a_minus-1)/a_minus


c      set i.c. for H(t) and T(t):
c     --------------------------
      do tt=1,first-1
         H(tt)=0.
         T(tt)=0.
      end do
      do tt=1,(first-1)*30
         H_day(tt)=0.
         T_day(tt)=.27
      end do
      nterms=5
      do ii=1,nterms
         terms(ii)=0.0
      end do
      H(first)=0.0
      T(first)=.1


c     solve for H(t) and T(t):
c     -----------------------
      IFAIL_nag=0
      X_nag=first
      XEND_nag=last
      Y_nag(1)=T(first)
cc      Y_nag(2)=H(first)
      ARELAB_nag='R'
      call D02CJF(X_nag, XEND_nag, N_nag, Y_nag, FCN_nag, TOL_nag
     $     , ARELAB_nag, OUTPUT_nag, D02CJW, W_nag, IFAIL_nag)

c     The block below now set to print H and  T
c     -------------------------------------------
      do ii=first*30,(last-1)*30
cc         t_middle=int((ii-1.)*30+16)
cc         tday=int(30.*mod(ii-1,12))+15
         tday=int(1.*mod(ii-1,360))+1
         write(10,'( i10, f14.6, f14.6, f14.6)')
     *   ii, T_day(ii), H_day(ii), seasonal_b0(tday)
cc         ,H_day(t_middle),seasonal_b0(tday)
cc     *        ,HW1(t_middle),HW2(t_middle),HW3(t_middle)
      enddo

      stop
      end


      SUBROUTINE OUTPUT_nag(XSOL, Y)
c     called by Nag routine D02CJF to obtain the output at intervals
c     specified by XSOL.
      include 'mm_standard.h'
*Eli: 20010804:  added dimension of F, see below for use in calling FNC_nag:
      real*8   XSOL, Y(N_nag), F(N_nag)
      real*8 afunch_rms,h_avg
      integer tt,PrintYear,tyear,iavg, tday,month
c      write(*,*) " OUTPUT_nag, XSOL=",XSOL

*Eli: 20010804:
c     added a call to fcn_nag, to make sure that the thermocline depth
c     and other terms that are evaluated there and used in output_nag
c     are indeed evaluated at the time of call to output_nag:
      call FCN_nag(XSOL, Y, F)
      
c     time is mesured in months.  get solution at a 
c     day resolution:

      tt=int((xsol-1.)*30.+1.)
      T_day(tt)=Y(1)
      H_day(tt)=terms(1)
cc      HW1(tt)=terms(2)
cc     HW2(tt)=terms(3) 
cc      HW3(tt)=terms(4) 
c     determine time of next output from Nag routine:
      if(abs(XSOL-anint(XSOL)).le.1E-7) XSOL=anint(XSOL)
      if(mod(XSOL,12000.0).le.1E-4) print*,'year',int(XSOL)/12
      xsol=xsol+1./30.
      XSOLp=XSOL
      if (xsol.ge.last) xsol=last
      return
      end


      SUBROUTINE FCN_nag(X, Y, F)
c     called by Nag routine D02CJF to evaluate the rhs of the
c     differential equations to be solved. 
      include 'mm_standard.h'
      real*8   X,Y(N_nag),F(N_nag),h_interpolated,RR,b,hE,hE1
     *         , TT, HH, Tsub, b_1,b_2,tmp,Rand
     *        ,TT_delay1,HH_delay,TT_delay2,W_var,rW_var,w1,w2
      integer  tday,tmon,tday2,tday_total,tday1,tday3
      common/Tsub1/ Tsub,he
c      write(*,*) " FCN_nag, XSOL=",X

c     month depended kappa:
c     --------------------
      tmon=int(X)
      tday_total=int((X-1.)*30.+1.)
      tday=int(30.*mod(X-1,12.0))+1
      if (tday .gt. 360) tday=360
      tday1=int(30.*mod(X+12-delay1-1,12.0))+1
      tday2=int(30.*mod(X+12-delay2-1,12.0))+1
      tday3=int(30.*mod(X+12-delay3-1,12.0))+1
c-----imply the seasonal cycle to mean fields
      b=b0*seasonal_b0(tday)
      b_1=b0*seasonal_b0(tday1)
      b_2=b0*seasonal_b0(tday2)
      W_var=W0*seasonal_W0(tday)
      rW_var=rW*seasonal_rW(tday3)

      TT=Y(1)
c     set solutions at a time delay:
c     -----------------------------
      w1=X*30.0-int(X*30.0)
      w2=1.-w1
      TT_delay1= w1*T_day(int(tday_total-delay1*30)  )
     *          +w2*T_day(int(tday_total-delay1*30)+1)
      TT_delay2= w1*T_day(int(tday_total-delay2*30)  )
     *          +w2*T_day(int(tday_total-delay2*30)+1)
      HH_delay = w1*H_day(int(tday_total-delay4*30)  )
     *          +w2*H_day(int(tday_total-delay4*30)+1)
      Rand =     w1*random_amp(tday_total)
     *          +w2*random_amp(tday_total+1)
c     calculationg h-east out of h-east and T-east:
c     --------------------------------------------
      hE1=em21*rW_var*(em11*rE*HH_delay
     *   -dT*em12*tau1/(betta*ru)*A*b_1*TT_delay1)
      hE=hE1+ dT*tau2/(ru*Co)*b_2*TT_delay2*em22
      terms(1)=hE
cc      terms(2)=em21*rW_var*em11*rE*HH_delay
cc      terms(3)=em21*rW_var*dT*em12*tau1/(betta*ru)*A*b_1*TT_delay1
cc      terms(4)=dT*tau2/(ru*Co)*b_2*TT_delay2*em22
cc      terms(3)=DT*em2*0.125*b_2*TT_delay2

c     hyperbolic function for Tsub (Cane & Zebiak 1987):
c     -------------------------------------------------
      call Tsub_calc(Tsub,hE)

c     righthandside of equations:
c-----------------------------------------------------------
      F(1)=   - et*TT - gamma*W_var/H1*(TT-Tsub) +Rand*0.0
c      terms(2)=Tsub
c  linear version 1
cc      F(1)=   -et*TT - gamma*W0/H1*(TT-.25*hE) +Rand*0.0
c-----------------------------------------------------------
c     linear version 2
cc      F(1)=   -0.5*TT + 0.01875*hE
      return
      end


      SUBROUTINE coupling()

      include 'mm_standard.h'
      integer tt
      real*8 pi
      pi=acos(-1.)
      do tt=1,360
         seasonal_b0(tt)= 1+var_b0
     * *cos(2*pi*(tt-1-30.*month_max_b0+15.)/360.)
         seasonal_W0(tt)= 1+var_W0
     * *cos(2*pi*(tt-1-30.*month_max_W0+15.)/360.)
         seasonal_rW(tt)= 1+var_rW
     * *cos(2*pi*(tt-1-30.*month_max_rW+15.)/360.)
      enddo
      if (shape.eq.2) then
      do tt=1,30
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(1)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(1)
      enddo
      do tt=31,60
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(2)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(2)
      enddo
      do tt=61,90
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(3)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(3)
      enddo
      do tt=91,120
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(4)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(4)
      enddo
      do tt=121,150
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(5)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(5)
      enddo
      do tt=151,180
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(6)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(6)
      enddo
      do tt=181,210
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(7)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(7)
      enddo
      do tt=211,240
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(8)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(8)
      enddo
      do tt=241,270
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(9)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(9)
      enddo
      do tt=271,300
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(10)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(10)
      enddo
      do tt=301,330
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(11)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(11)
      enddo
      do tt=331,360
         seasonal_b0(tt)= 1.+var_b0*var_b0_dec(12)
         seasonal_W0(tt)= 1.+var_W0*var_W0_dec(12)
      enddo
      endif

      return 
      end


      SUBROUTINE Tsub_calc(Tsub,heE)

      include 'mm_standard.h'
      real*8 a_plus,a_minus,b_plus,b_minus,hh,h_plus,h_minus
      real*8 Tsub,heE,k, T1, T2, b1, b2, H0
      integer ver

      common/Tsub_coef/ k,b_plus,b_minus,a_plus,a_minus,h_plus,h_minus

      T1=28.
      T2=-40.
      b1=1./(80.)
      b2=1./(33.)
      H0=47.

      ver=2
      if (ver.eq.1) then
c     hyperbolic function for Tsub (Cane & Zebiak 1987):
c     -------------------------------------------------
      if(heE.ge.0.0) then
         Tsub=T1*( tanh(b1*(H0+heE)) - tanh(b1*H0) )
      else
         Tsub=T2*( tanh(b2*(H0-heE)) - tanh(b2*H0) )
      endif

      else
c     hyperbolic function for Tsub (Munnich,Cane&Zebiak 1991):
c     -------------------------------------------------
      hh=k*heE
      if(hh.lt.h_minus) then
         Tsub=-b_minus+b_minus/a_minus*(tanh(a_minus/b_minus
     *        *(hh-h_minus))+1)
      elseif (hh.ge.h_minus.and.hh.lt.h_plus) then
         Tsub=hh
      else
         Tsub=b_plus+b_plus/a_plus*(tanh(a_plus/b_plus*(hh-h_plus))-1)
      endif

      endif 

      return 
      end
